﻿using System;
using System.Collections.Generic;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Options;
using Pomelo.EntityFrameworkCore.MySql.Scaffolding.Internal;

namespace Dardabahn_WPF.Models;

public partial class DardabahnContext : DbContext
{
    public DardabahnContext()
    {
    }

    public DardabahnContext(DbContextOptions<DardabahnContext> options)
        : base(options)
    {
    }
    public virtual DbSet<Auto> Autos { get; set; }

    public virtual DbSet<Mitglied> Mitglieds { get; set; }

    public virtual DbSet<Rennen> Rennens { get; set; }

    public virtual DbSet<Rennstrecke> Rennstreckes { get; set; }

    public virtual DbSet<Team> Teams { get; set; }

    public virtual DbSet<Teilnahme> Teilnahmes { get; set; }

    public virtual DbSet<Veranstaltung> Veranstaltungs { get; set; }

    protected override void OnConfiguring(DbContextOptionsBuilder optionsBuilder)
#warning To protect potentially sensitive information in your connection string, you should move it out of source code. You can avoid scaffolding the connection string by using the Name= syntax to read it from configuration - see https://go.microsoft.com/fwlink/?linkid=2131148. For more guidance on storing connection strings, see https://go.microsoft.com/fwlink/?LinkId=723263.
        => optionsBuilder.UseMySql("server=192.168.137.76;user=philipp;password=1234;database=dardabahn", Microsoft.EntityFrameworkCore.ServerVersion.Parse("10.4.32-mariadb"));

    protected override void OnModelCreating(ModelBuilder modelBuilder)
    {
        modelBuilder
            .UseCollation("utf8mb4_general_ci")
            .HasCharSet("utf8mb4");

        modelBuilder.Entity<Auto>(entity =>
        {
            entity.HasKey(e => e.AutoId).HasName("PRIMARY");

            entity.ToTable("auto");

            entity.HasIndex(e => e.TeamId, "Team_ID");

            entity.Property(e => e.AutoId)
                .HasColumnType("int(11)")
                .HasColumnName("Auto_ID");
            entity.Property(e => e.Farbe).HasMaxLength(255);
            entity.Property(e => e.TeamId)
                .HasColumnType("int(11)")
                .HasColumnName("Team_ID");

            entity.HasOne(d => d.Team).WithMany(p => p.Autos)
                .HasForeignKey(d => d.TeamId)
                .HasConstraintName("auto_ibfk_1");
        });

        modelBuilder.Entity<Mitglied>(entity =>
        {
            entity.HasKey(e => e.MitgliedId).HasName("PRIMARY");

            entity.ToTable("mitglied");

            entity.HasIndex(e => e.TeamId, "Team_ID");

            entity.Property(e => e.MitgliedId)
                .HasColumnType("int(11)")
                .HasColumnName("Mitglied_ID");
            entity.Property(e => e.Alter).HasColumnType("int(11)");
            entity.Property(e => e.Nachname).HasMaxLength(255);
            entity.Property(e => e.TeamId)
                .HasColumnType("int(11)")
                .HasColumnName("Team_ID");
            entity.Property(e => e.Vorname).HasMaxLength(255);

            entity.HasOne(d => d.Team).WithMany(p => p.Mitglieds)
                .HasForeignKey(d => d.TeamId)
                .HasConstraintName("mitglied_ibfk_1");
        });

        modelBuilder.Entity<Rennen>(entity =>
        {
            entity.HasKey(e => e.RennenId).HasName("PRIMARY");

            entity.ToTable("rennen");

            entity.HasIndex(e => e.TeilnahmeId, "Teilnahme_ID");

            entity.Property(e => e.RennenId)
                .HasColumnType("int(11)")
                .HasColumnName("Rennen_ID");
            entity.Property(e => e.Endzeit).HasColumnType("float(5,3)");
            entity.Property(e => e.Startzeit).HasColumnType("float(5,3)");
            entity.Property(e => e.Szenarien).HasMaxLength(255);
            entity.Property(e => e.TeilnahmeId)
                .HasColumnType("int(11)")
                .HasColumnName("Teilnahme_ID");

            entity.HasOne(d => d.Teilnahme).WithMany(p => p.Rennens)
                .HasForeignKey(d => d.TeilnahmeId)
                .HasConstraintName("rennen_ibfk_1");
        });

        modelBuilder.Entity<Rennstrecke>(entity =>
        {
            entity.HasKey(e => e.RennstreckenId).HasName("PRIMARY");

            entity.ToTable("rennstrecke");

            entity.HasIndex(e => e.TeamId, "Team_ID");

            entity.Property(e => e.RennstreckenId)
                .HasColumnType("int(11)")
                .HasColumnName("Rennstrecken_ID");
            entity.Property(e => e.TeamId)
                .HasColumnType("int(11)")
                .HasColumnName("Team_ID");

            entity.HasOne(d => d.Team).WithMany(p => p.Rennstreckes)
                .HasForeignKey(d => d.TeamId)
                .HasConstraintName("rennstrecke_ibfk_1");
        });

        modelBuilder.Entity<Team>(entity =>
        {
            entity.HasKey(e => e.TeamId).HasName("PRIMARY");

            entity.ToTable("team");

            entity.Property(e => e.TeamId)
                .HasColumnType("int(11)")
                .HasColumnName("Team_ID");
            entity.Property(e => e.Name).HasMaxLength(255);
        });

        modelBuilder.Entity<Teilnahme>(entity =>
        {
            entity.HasKey(e => e.TeilnahmeId).HasName("PRIMARY");

            entity.ToTable("teilnahme");

            entity.HasIndex(e => e.TeamId, "Team_ID");

            entity.HasIndex(e => e.VeranstaltungsId, "Veranstaltungs_ID");

            entity.Property(e => e.TeilnahmeId)
                .HasColumnType("int(11)")
                .HasColumnName("Teilnahme_ID");
            entity.Property(e => e.Datum).HasColumnType("datetime");
            entity.Property(e => e.TeamId)
                .HasColumnType("int(11)")
                .HasColumnName("Team_ID");
            entity.Property(e => e.VeranstaltungsId)
                .HasColumnType("int(11)")
                .HasColumnName("Veranstaltungs_ID");

            entity.HasOne(d => d.Team).WithMany(p => p.Teilnahmes)
                .HasForeignKey(d => d.TeamId)
                .HasConstraintName("teilnahme_ibfk_1");

            entity.HasOne(d => d.Veranstaltungs).WithMany(p => p.Teilnahmes)
                .HasForeignKey(d => d.VeranstaltungsId)
                .HasConstraintName("teilnahme_ibfk_2");
        });

        modelBuilder.Entity<Veranstaltung>(entity =>
        {
            entity.HasKey(e => e.VeranstaltungsId).HasName("PRIMARY");

            entity.ToTable("veranstaltung");

            entity.HasIndex(e => e.RennstreckenId, "Rennstrecken_ID");

            entity.Property(e => e.VeranstaltungsId)
                .HasColumnType("int(11)")
                .HasColumnName("Veranstaltungs_ID");
            entity.Property(e => e.RennstreckenId)
                .HasColumnType("int(11)")
                .HasColumnName("Rennstrecken_ID");

            entity.HasOne(d => d.Rennstrecken).WithMany(p => p.Veranstaltungs)
                .HasForeignKey(d => d.RennstreckenId)
                .HasConstraintName("veranstaltung_ibfk_1");
        });

        OnModelCreatingPartial(modelBuilder);
    }

    partial void OnModelCreatingPartial(ModelBuilder modelBuilder);
}
