﻿using Dardabahn_WPF.Controller;
using Dardabahn_WPF.Models;
using Microsoft.EntityFrameworkCore.Migrations.Operations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.Xaml;

namespace Dardabahn_WPF
{
    /// <summary>
    /// Interaktionslogik für Rennen_Fenster.xaml
    /// </summary>
    public partial class Rennen_Fenster : Window
    {
        NoTeilnahme noTeilnahme = null;
        IO_Controller ioc;
        int id;
        public Rennen_Fenster(IO_Controller ioc, int id)
        {
            InitializeComponent();
            this.ioc = ioc;
            this.id = id;
            UpdateData();
        }

        private void Button_Gewinn1_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                // Hole alle Rennen des ausgewählten Events
                var rennenList = ioc.rc.ReadAllAnzeigebyVeranstaltung(id);

                // Gruppiere Rennen nach TeamId
                var gruppiertNachTeam = rennenList
                    .GroupBy(r => r.TeamId)
                    .ToList();

                // Dictionary zum Speichern der Mittelwerte pro Team
                Dictionary<int?, float?> teamMittelwerte = new Dictionary<int?, float?>();

                // Iteriere über jedes Team
                foreach (var teamGroup in gruppiertNachTeam)
                {
                    // Sortiere die Rennen eines Teams nach Rundenzeit (aufsteigend)
                    var sortierteRennen = teamGroup
                        .Where(r => r.Rundenzeit > 0) // Filtere nur gültige Zeiten
                        .OrderBy(r => r.Rundenzeit)
                        .Take(3) // Nimm die 3 schnellsten Rennen
                        .ToList();

                    // Berechne den Mittelwert, wenn mindestens 3 Rennen vorhanden sind
                    if (sortierteRennen.Count == 3)
                    {
                        float? mittelwert = sortierteRennen.Average(r => r.Rundenzeit);
                        teamMittelwerte[teamGroup.Key] = mittelwert;
                    }
                }

                // Überprüfe, ob Mittelwerte berechnet wurden
                if (teamMittelwerte.Count > 0)
                {
                    // Bestimme das Team mit dem niedrigsten Mittelwert
                    var gewinner = teamMittelwerte.OrderBy(kvp => kvp.Value).First();

                    MessageBox.Show($"Team: {gewinner.Key} hat gewonnen mit einem Mittelwert von {gewinner.Value:F2} Sekunden!!! :)");
                }
                else
                {
                    MessageBox.Show("KEINER HAT GEWONNEN!!! :)");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Fehler: {ex.Message}");
            }
        }


        private void Button_Gewinn2_Click(object sender, RoutedEventArgs e)
        {

            RennenAnzeige winner = null;
            List<RennenAnzeige> rennlist = new List<RennenAnzeige>();

            foreach (var x in ioc.rc.ReadAllAnzeigebyVeranstaltung(id))
            {
                if (x.Szenarien == String.Empty || x.Szenarien == null || x.Szenarien == "")
                {
                    rennlist.Add(x);
                }

            }
            if (rennlist.Count < 2)
            {
                MessageBox.Show("KEINER HAT GEWONNEN!!! :)");
            }
            else
            {
                try
                {
                    rennlist = rennlist.OrderByDescending(x => x.Rundenzeit).ToList();
                    winner = rennlist[1];

                    MessageBox.Show($"Team: {winner.TeamId} hat gewonnen, weil es die zweit langsamste Zeit ({winner.Rundenzeit} sek) hat!!! :)");
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }

        }

        private void Button_Gewinn3_Click(object sender, RoutedEventArgs e)
        {
            RennenAnzeige winner = null;
            List<RennenAnzeige> dnfList = new List<RennenAnzeige>();

            foreach (var x in ioc.rc.ReadAllAnzeigebyVeranstaltung(id))
            {
                if (x.Szenarien == "DNF")
                {
                    dnfList.Add(x);
                }
            }
            if (dnfList.Count < 1)
            {
                MessageBox.Show("KEINER HAT GEWONNEN!!! :)");
            }
            else
            {
                try
                {
                    dnfList = dnfList.OrderByDescending(x => x.Startzeit).ToList();
                    winner = dnfList[0];

                    MessageBox.Show($"Team: {winner.TeamId} hat gewonnen, weil es DNF hat und die langsamste Zeit ({winner.Startzeit} sek) hat!!! :)");
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
        }
        private void NoTeilnahmeBtn_Click(object sender, RoutedEventArgs e)
        {
            // Der auch von Philipp (weil gehört zu NoTeilnahme)
            try {
                if (noTeilnahme == null) {
                    noTeilnahme = new NoTeilnahme(ioc,id);
                    noTeilnahme.Show();
                }
                else {
                    noTeilnahme.Close();
                    noTeilnahme = new NoTeilnahme(ioc,id);
                    noTeilnahme.Show();
                }

            }
            catch {
                MessageBox.Show("Fenster konnte nicht geöffnet werden!");
            }
        }
        public void UpdateData()
        {
            DataGrid_Rennen.ItemsSource = null;
            List<RennenAnzeige> Rennen = ioc.rc.ReadAllAnzeigebyVeranstaltung(id);
            List<RennenAnzeige> AnzeigRennen = new List<RennenAnzeige>();
            foreach (RennenAnzeige x in Rennen)
            {
                if (x.Szenarien == "FR" || x.Szenarien == "DNS")
                {
                    x.Rundenzeit = 0;
                    x.Startzeit = 0;
                    x.Endzeit = 0;
                    AnzeigRennen.Add(x);
                }
                else if (x.Szenarien == "DNF")
                {
                    x.Endzeit = 0;
                    x.Rundenzeit = 0;
                    AnzeigRennen.Add(x);
                }
                else if (x.Szenarien == "")
                {
                    AnzeigRennen.Add(x);
                }
            }
            DataGrid_Rennen.ItemsSource = AnzeigRennen;
        }


    }
}
