import time
import pigpio
import sys
from oled_display import update_oled_display  # OLED-Display-Funktion importieren
import mysql.connector

# /home/PhilippPI/Documents/RST/Dardabahn/DB_Connector/bin   die Route zu activate

# GPIO-Pins für Sensor und LEDs
SENSOR_PIN = 22
LED_RED = 26
LED_YELLOW = 19
LED_GREEN = 13
BUTTON_PIN = 6

# Globale Variablen
global VeranstaltungsID
global Teamname
global Szenario
VeranstaltungsID = 0
Teamname = ""
Szenario = ""
green_light_started = False
early_start_detected = False
program_running = False  # Programm startet erst nach Button-Druck
start_time = None  # Zeit zwischen Grün und erstem Sensorereignis
end_time = None  # Zeit zwischen Grün und zweitem Sensorereignis
green_light_time = None  # Startzeit der Grünphase
interrupt_count = 0  # Anzahl der Sensorereignisse
last_trigger_time = 0  # Zeit des letzten Sensorereignisses
first_sensor_time = None  # Zeitpunkt, an dem der erste Sensor überquert wurde
BOUNCE_TIME = 0.5  # Entprellzeit in Sekunden
DNS_TIMEOUT = 5  # Zeit in Sekunden, bevor DNS ausgelöst wird
DNF_TIMEOUT = 5  # Zeit in Sekunden, bevor DNF ausgelöst wird

def DB_Auswahl_Veranstaltung():
    global VeranstaltungsID
    VeranstaltungsID = int(input("Bitte geben sie eine VeranstaltungsID ein:\n"))
    if VeranstaltungsID <= 0:
        DB_Auswahl_Veranstaltung()

def DB_Auswahl_Teamname():
    global Teamname
    Teamname = input("Bitte geben sie einen Teamnamen ein:\n")
    if Teamname == "":
        DB_Auswahl_Teamname()


def DB_Get_Teilnahme_ID(Teamname, VeranstaltungsID):
    """
    Holt die Teilnahme_ID aus der Datenbank basierend auf Teamname und VeranstaltungsID.
    """
    db_connection = mysql.connector.connect(
        host="localhost",
        user="philipp",
        passwd="1234",
        database="dardabahn"
    )
    cursor = db_connection.cursor()
    try:
        query = """
        SELECT tn.Teilnahme_ID 
        FROM teilnahme tn 
        JOIN team t ON tn.Team_ID = t.Team_ID 
        WHERE t.Name = %s AND tn.Veranstaltungs_ID = %s
        """
        values = (Teamname, VeranstaltungsID)
        cursor.execute(query, values)
        result = cursor.fetchone()
        if result:
            # print(f"Gefundene Teilnahme_ID: {result[0]}")  # Debug-Ausgabe
            return result[0]  # Teilnahme_ID
        else:
            print("Keine passende Teilnahme_ID gefunden.")  # Debug-Ausgabe
            return None
    except mysql.connector.Error as err:
        print(f"Fehler bei der Datenbankabfrage: {err}")
        return None
    finally:
        cursor.close()
        db_connection.close()



def DB_Eingabe():
    """
    Führt die Eingabe in die Datenbank durch.
    """
    update_oled_display("DB", small=True)
    answer = input("Wollen Sie die Daten speichern?\n[Y] [N]\n")
    if answer.upper() == "Y":
        db_connection = mysql.connector.connect(
            host="localhost",
            user="philipp",
            passwd="1234",
            database="dardabahn"
        )
        cursor = db_connection.cursor()
        try:
            # Debug: Variablen anzeigen
            # print(f"Teamname: {Teamname}, VeranstaltungsID: {VeranstaltungsID}")

            # Teilnahme_ID abrufen
            teilnahme_id = DB_Get_Teilnahme_ID(Teamname, VeranstaltungsID)
            if teilnahme_id is None:
                print("Speichern abgebrochen, Teilnahme_ID nicht gefunden.")
                return
            
            # Debug: Teilnahme_ID anzeigen
            # print(f"Teilnahme_ID: {teilnahme_id}")
            
            if Szenario == "DNF":
                Endzeit = 0
            elif Szenario == "DNS" or Szenario == "FR":
                Startzeit = 0; Endzeit = 0
            # Daten in die Tabelle 'rennen' einfügen
            query = """
            INSERT INTO rennen (Startzeit, Endzeit, Szenarien, Teilnahme_ID) 
            VALUES (%s, %s, %s, %s)
            """
            values = (start_time, end_time, Szenario, teilnahme_id)
            cursor.execute(query, values)
            db_connection.commit()  # Änderungen speichern
            print("Daten erfolgreich gespeichert.")
        except mysql.connector.Error as err:
            print(f"Fehler bei der Datenbankeingabe: {err}")
        finally:
            cursor.close()
            db_connection.close()
    else:
        print("Speichern abgebrochen.")



# Funktion zur Initialisierung von pigpio
def initialize_pi():
    """Initialisiert die Verbindung zur pigpio-Daemon."""
    pi = pigpio.pi()
    if not pi.connected:
        print("Fehler: Keine Verbindung zu pigpio-Daemon.")
        sys.exit()
    return pi

# Funktion zur Steuerung der Ampel
def start_traffic_light(pi):
    """Ampelsteuerung: Von Rot auf Gelb auf Grün."""
    global green_light_started, early_start_detected, green_light_time

    print("Start Ampel...")
    update_oled_display("Rot")

    # Rot an
    pi.write(LED_RED, 1)
    pi.write(LED_YELLOW, 0)
    pi.write(LED_GREEN, 0)
    time.sleep(1.5)

    if early_start_detected:
        return

    # Gelb an
    pi.write(LED_RED, 0)
    pi.write(LED_YELLOW, 1)
    update_oled_display("Gelb")
    time.sleep(1.5)

    if early_start_detected:
        return

    # Grün an
    pi.write(LED_YELLOW, 0)
    pi.write(LED_GREEN, 1)
    update_oled_display("Start")
    green_light_started = True
    green_light_time = time.time()
    print("Rennen-Start")

# Funktion zur Verarbeitung von Sensorereignissen
def process_sensor_event():
    global green_light_started, start_time, end_time, first_sensor_time, interrupt_count

    if green_light_started:
        current_time = time.time() - green_light_time
        if interrupt_count == 0:
            start_time = current_time
            first_sensor_time = time.time()  # Zeitpunkt des ersten Sensors
        elif interrupt_count == 1:
            end_time = current_time

        interrupt_count += 1

# Callback-Funktion für den Sensor
def sensor_callback(pin, level, tick):
    """Kurzer Interrupt-Handler für Sensorereignisse."""
    global green_light_started, early_start_detected, last_trigger_time

    current_time = time.time()
    if (current_time - last_trigger_time) < BOUNCE_TIME:
        return

    last_trigger_time = current_time
    global program_running
    # Frühstart prüfen
    if not green_light_started and program_running:
        global Szenario
        Szenario = "FR"
        early_start_detected = True
        print("Frühstart erkannt!")
        update_oled_display("FS")
        pi.write(LED_GREEN, 0)
        for _ in range(0, 5):
            time.sleep(.2)
            pi.write(LED_RED, 1)
            pi.write(LED_YELLOW, 1)
            pi.write(LED_GREEN, 1)
            time.sleep(.2)
            pi.write(LED_RED, 0)
            pi.write(LED_YELLOW, 0)
            pi.write(LED_GREEN, 0)
        stop_program()
        return

    # Sensorereignis verarbeiten
    process_sensor_event()

# Funktion zur Berechnung der Gesamtzeit
def calculate_and_display_times():
    """Berechnet und zeigt die Startzeit, Endzeit und Gesamtzeit an."""
    global start_time, end_time

    if start_time is not None and end_time is not None:
        total_time = end_time - start_time
        print(f"Startzeit: {start_time:.3f} Sekunden.")
        print(f"Endzeit: {end_time:.3f} Sekunden.")
        print(f"Gesamtzeit: {total_time:.3f} Sekunden.")
        update_oled_display(f"{total_time:.3f}s", small=True)
        time.sleep(3)

# Funktion zur Prüfung auf DNS
def check_for_dns():
    """Prüft, ob Did Not Start (DNS) ausgelöst werden soll."""
    global green_light_started, green_light_time, interrupt_count

    if green_light_started and interrupt_count == 0:
        elapsed_time = time.time() - green_light_time
        if elapsed_time > DNS_TIMEOUT:
            global Szenario
            Szenario = "DNS"
            print("Did Not Start (DNS) erkannt!")
            update_oled_display("DNS")
            pi.write(LED_GREEN, 0)
            for _ in range(0, 5):
                time.sleep(.2)
                pi.write(LED_RED, 1)
                pi.write(LED_YELLOW, 1)
                pi.write(LED_GREEN, 1)
                time.sleep(.2)
                pi.write(LED_RED, 0)
                pi.write(LED_YELLOW, 0)
                pi.write(LED_GREEN, 0)
            stop_program()


            

# Funktion zur Prüfung auf DNF
def check_for_dnf():
    """Prüft, ob Did Not Finish (DNF) ausgelöst werden soll."""
    global first_sensor_time, interrupt_count

    if first_sensor_time is not None and interrupt_count == 1:
        elapsed_time = time.time() - first_sensor_time
        if elapsed_time > DNF_TIMEOUT:
            global Szenario
            Szenario = "DNF"
            print("Did Not Finish (DNF) erkannt!")
            update_oled_display("DNF")
            pi.write(LED_GREEN, 0)
            for _ in range(0, 5):
                time.sleep(.2)
                pi.write(LED_RED, 1)
                pi.write(LED_YELLOW, 1)
                pi.write(LED_GREEN, 1)
                time.sleep(.2)
                pi.write(LED_RED, 0)
                pi.write(LED_YELLOW, 0)
                pi.write(LED_GREEN, 0)
            stop_program()


# Funktion zum Beenden des Programms
def stop_program():
    """Beendet das Programm sauber."""
    global pi, program_running, callback

    if not program_running:
        return

    program_running = False
    callback.cancel()

    pi.write(LED_RED, 0)
    pi.write(LED_YELLOW, 0)
    pi.write(LED_GREEN, 0)
    pi.stop()

# Funktion zur Verarbeitung des Button-Drückens
def button_callback(pin, level, tick):
    """Callback-Funktion für den Button."""
    global program_running
    if not program_running:
        print("Button gedrückt! Programm wird gestartet...")
        update_oled_display("Bereit!")
        program_running = True  # Setze Flag, um Hauptprogramm zu starten

# Funktion zum Warten auf den Button-Drücken
def wait_for_button():
    """Wartet, bis der Button gedrückt wird."""
    global program_running
    print("Warte auf Button-Druck...")
    update_oled_display("Warte")
    while not program_running:
        time.sleep(0.1)

# Hauptlaufzeitlogik
def main_loop():
    """Hält das Programm am Laufen, bis alle Sensorereignisse abgeschlossen sind."""
    global program_running, interrupt_count
    try:
        while program_running:
            if interrupt_count == 2:
                calculate_and_display_times()
                stop_program()
            check_for_dns()  # DNS-Überprüfung
            check_for_dnf()  # DNF-Überprüfung
            time.sleep(0.1)
    except KeyboardInterrupt:
        stop_program()


# Initialisierung von pigpio
pi = initialize_pi()
# Sensor-Pin
pi.set_mode(SENSOR_PIN, pigpio.INPUT)
# LED-Pins
pi.set_mode(LED_RED, pigpio.OUTPUT)
pi.set_mode(LED_YELLOW, pigpio.OUTPUT)
pi.set_mode(LED_GREEN, pigpio.OUTPUT)
# Button-Pin
pi.set_mode(BUTTON_PIN, pigpio.INPUT)
pi.set_pull_up_down(BUTTON_PIN, pigpio.PUD_DOWN)
# Sensor-Callback setzen
callback = pi.callback(SENSOR_PIN, pigpio.RISING_EDGE, sensor_callback)
# Button-Callback setzen
pi.callback(BUTTON_PIN, pigpio.RISING_EDGE, button_callback)

# Hauptprogramm erst starten, wenn Button gedrückt wurde
try:
    DB_Auswahl_Veranstaltung()
    DB_Auswahl_Teamname()
    wait_for_button()  # Warten, bis Button gedrückt wird
    start_traffic_light(pi)  # Ampelsteuerung starten
    main_loop()  # Hauptprogramm ausführen
    DB_Eingabe()
except KeyboardInterrupt:
    stop_program()
finally:
    sys.exit()